(*
 * Graph: generic graph library
 * Copyright (C) 2004
 * Sylvain Conchon, Jean-Christophe Filliatre and Julien Signoles
 * 
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License version 2, as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * See the GNU Library General Public License version 2 for more details
 * (enclosed in the file LGPL).
 *)

(* $Id: ford_fulkerson.mli,v 1.1 2004/02/09 09:29:27 conchon Exp $ *)

(**  Ford-Fulkerson maximal flow algorithm *)

module type FLOW = sig
  type label
  type t

  (** maximum and minimum capacities for a label on an edge *)

  val max_capacity : label -> t
  val min_capacity : label -> t

  (** current flow for a label on an edge *)

  val flow : label -> t

  (** [+] and [-] on flows. *)

  val add : t -> t -> t
  val sub : t -> t -> t

  (** neutral element for [add] and [remove]. *)

  val zero : t

  (** maximal and minimal flows *)

  val max : t
  val min : t

  (** a total ordering over flows *)

  val compare : t -> t -> int

end

module type G = sig
  type t

  module V : Sig.HASHABLE

  module E : sig
    type t
    type label
    val src : t -> V.t
    val dst : t -> V.t
    val label : t -> label
  end

  val iter_succ_e : (E.t -> unit) -> t -> V.t -> unit
  val iter_pred_e : (E.t -> unit) -> t -> V.t -> unit
end

module Make(G: G)(F: FLOW with type label = G.E.label) :
sig

  val maxflow : G.t -> G.V.t -> G.V.t -> (G.V.t -> G.V.t -> F.t) * F.t
    (** [maxflow g v1 v2] searchs the maximal flow from [v1] to [v2]
        using the Ford-Fulkerson algorithm.  It returns the new flows
        on each edges and the growth of the flow. *)
    
end
