(*
 * Graph: generic graph library
 * Copyright (C) 2004
 * Sylvain Conchon, Jean-Christophe Filliatre and Julien Signoles
 * 
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License version 2, as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * See the GNU Library General Public License version 2 for more details
 * (enclosed in the file LGPL).
 *)

module type FLOW = sig
  type label
  type t
  val flow : label -> t
  val add : t -> t -> t
  val sub : t -> t -> t
  val zero : t
  val compare : t -> t -> int
  val min : t -> t -> t
end

module type G = sig
  type t
  module V : Sig.COMPARABLE
  module E : sig 
    type t 
    type label 
    val compare : t -> t -> int
    val create : V.t -> label -> V.t -> t
    val label : t -> label 
    val dst : t -> V.t 
    val src : t -> V.t
  end 
  val nb_vertex : t -> int
  val iter_vertex : (V.t -> unit) -> t -> unit
  val iter_edges : (V.t -> V.t -> unit) -> t -> unit
  val fold_pred : (V.t -> 'a -> 'a) -> t -> V.t -> 'a -> 'a
  val fold_succ_e : (E.t -> 'a -> 'a) -> t -> V.t -> 'a -> 'a
  val fold_pred_e : (E.t -> 'a -> 'a) -> t -> V.t -> 'a -> 'a
end

module Make(G: G)(F: FLOW with type label = G.E.label) = 
struct
  
  module V = Hashtbl.Make(G.V)
  module E = Hashtbl.Make(Util.HTProduct(G.V)(G.V)) 
  module Se = Set.Make(G.E)
  module Sv = Set.Make(G.V)

  let excedents = V.create 997
  let hauteur = V.create 997
  let flot = E.create 997

  let fold_booleen f = List.fold_left (fun r x->(f x) or r) false

  let capacite_restante g e = F.sub (F.flow (G.E.label e)) (E.find flot (G.E.src e, G.E.dst e))

  let reste_excedent x = F.compare (V.find excedents x) F.zero > 0 
      
  let flux_et_reflux g x = 
    let s =  G.fold_succ_e 
	       (fun e s->if F.compare (capacite_restante g e) F.zero > 0 then e::s else s) 
	       g x [] 
    in 
      G.fold_pred_e 
	(fun e s -> 
	   if F.compare (E.find flot (G.E.src e, G.E.dst e)) F.zero > 0 
	   then (G.E.create (G.E.dst e) (G.E.label e) (G.E.src e))::s else s)
	g x s

  let pousser g e l =
    let x,y = G.E.src e , G.E.dst e in
    let ex = V.find excedents x in
    let cxy = capacite_restante g e in
      if F.compare ex F.zero > 0 &&
	F.compare cxy F.zero >0 &&
	V.find hauteur x = (V.find hauteur y + 1)
      then
	let d = F.min ex cxy in
	let fxy = E.find flot (x,y) in
	let ex = V.find excedents x in
	let ey = V.find excedents y in
	  E.replace flot (x,y) (F.add fxy d);
	  E.replace flot (y,x) (F.add d fxy);
	  V.replace excedents x (F.sub ex d);
	  V.replace excedents y (F.add ey d);
	  if reste_excedent x then l:=Sv.add x !l;
	  if reste_excedent y then l:=Sv.add y !l;
	  true
      else 
	(if F.compare ex F.zero > 0 then l:=Sv.add x !l; false)

  let elever g p x = 
    let u = flux_et_reflux g x in
      reste_excedent x
      && not (G.V.equal x p) 
      && List.for_all (fun e -> (V.find hauteur (G.E.src e)) <= (V.find hauteur (G.E.dst e))) u 
      && (let min = List.fold_left (fun m e-> min (V.find hauteur (G.E.dst e)) m) max_int u in
	    V.replace hauteur x (1+min); true)

  let init_preflot g s p = 
    G.iter_vertex (fun x -> V.add excedents x F.zero; V.add hauteur x 0) g;
    G.iter_edges (fun x y -> E.add flot (x,y) F.zero; E.add flot (x,y) F.zero) g;
    V.add hauteur s (G.nb_vertex g);
    G.fold_succ_e 
      (fun e l -> 
	 let y = G.E.dst e in
	   let c = F.flow (G.E.label e) in 
	     E.add flot (s,y) c;
	     E.add flot (y,s) (F.sub F.zero c);
	     V.add excedents y c;
	     y::l)
      g s []
      
  let maxflow g s p = 
    let push_and_pull l x = 
      G.fold_succ_e (fun e r->pousser g e l or r) g x false
      or G.fold_pred_e (fun e r->pousser g e l or r) g x false
    in
    let todo = ref (init_preflot g s p) in
      while 
	(fold_booleen (elever g p) !todo) or 
	(let l = ref Sv.empty in 
	 let r = fold_booleen (push_and_pull l) !todo in
	   todo:=Sv.elements !l; r)
      do () done;
      let flot_max = 
	G.fold_pred (fun y f -> F.add (E.find flot (y,p)) f) g p F.zero
      in (fun x y -> E.find flot (x,y)) , flot_max
end
