(*
 * Graph: generic graph library
 * Copyright (C) 2004
 * Sylvain Conchon, Jean-Christophe Filliatre and Julien Signoles
 * 
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License version 2, as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * See the GNU Library General Public License version 2 for more details
 * (enclosed in the file LGPL).
 *)

(* $Id: traverse.mli,v 1.9 2004/02/06 15:06:55 filliatr Exp $ *)

(** Graph traversal *)

module type G = sig
  type t
  module V : Sig.COMPARABLE
  val iter_vertex : (V.t -> unit) -> t -> unit
  val iter_succ : (V.t -> unit) -> t -> V.t -> unit
end

(** Depth-first search *)
module Dfs(G : G) : sig
  val iter : ?pre:(G.V.t -> unit) -> 
             ?post:(G.V.t -> unit) -> G.t -> unit
      (** [iter pre post g] visits all nodes of [g] in depth-first search, 
	 applying [pre] to each visited node before its successors,
	 and [post] after them. Each node is visited exactly once. *)

  val prefix : (G.V.t -> unit) -> G.t -> unit
      (** applies only a prefix function; note that this function is more
	efficient than [iter] *)

  val postfix : (G.V.t -> unit) -> G.t -> unit
      (** applies only a postfix function *)

  val iter_component : ?pre:(G.V.t -> unit) -> 
             ?post:(G.V.t -> unit) -> G.t -> G.V.t -> unit
  val prefix_component : (G.V.t -> unit) -> G.t -> G.V.t -> unit
  val postfix_component : (G.V.t -> unit) -> G.t -> G.V.t -> unit
      (** Same thing, but for a single connected component *)

  val has_cycle : G.t -> bool
      (** [has_cycle g] checks for a cycle in [g]. Linear in time and space. *)

end

(** Breadth-first search *)
module Bfs(G : G) : sig

  val iter : (G.V.t -> unit) -> G.t -> unit

  val iter_component : (G.V.t -> unit) -> G.t -> G.V.t -> unit

end


(** Graph traversal with marking. 
    Only applies to imperative graphs with marks. *)

module type GM = sig
  include G
  module Mark : sig
    val clear : t -> unit
    val get : V.t -> int
    val set : V.t -> int -> unit
  end
end

module Mark(G : GM) : sig

  val dfs : G.t -> unit
      (** [dfs g] traverses [g] in depth-first search, marking all nodes. *)

  val has_cycle : G.t -> bool
      (** [has_cycle g] checks for a cycle in [g]. Modifies the marks.
	  Linear time, constant space. *)

end

